/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/util/ClassCollector.java#2 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.util;
import openjava.mop.Environment;
import openjava.mop.OJClass;
import openjava.ptree.Expression;
import openjava.ptree.ParseTreeException;
import openjava.ptree.TypeName;
import openjava.ptree.Variable;
import openjava.ptree.util.ScopeHandler;

import java.util.HashSet;

/**
 * <code>ClassCollector</code> walks over an expression tree, calling {@link
 * openjava.ptree.Expression#getType} and {@link
 * openjava.ptree.Expression#getRowType} on each node, and forming a set of
 * distinct types.
 *
 * @author jhyde
 * @since 2 October, 2001
 * @version $Id: //guest/julian_hyde/saffron/src/main/saffron/util/ClassCollector.java#2 $
 **/
public class ClassCollector extends ScopeHandler
{
	private HashSet classSet = new HashSet();

	public ClassCollector(Environment env)
	{
		super(env);
	}

	private void register(OJClass clazz)
	{
		if (clazz == null) {
			return;
		}
		while (clazz.getComponentType() != null) {
			clazz = clazz.getComponentType();
		}
		classSet.add(clazz);
	}

	public TypeName evaluateDown(TypeName p) throws ParseTreeException
	{
		Environment env = getEnvironment();
	    OJClass clazz = env.lookupClass( env.toQualifiedName( p.getName() ) );
		register(clazz);
		return super.evaluateDown(p);
	}

	public OJClass[] getClasses()
	{
		return (OJClass[]) classSet.toArray(new OJClass[0]);
	}
}

// End ClassCollector.java
